<?php

namespace Drupal\commerce_add_to_cart_confirmation\Element;

use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Render\Element\RenderElement;
use Drupal\Core\Render\PlaceholderGeneratorInterface;
use Drupal\views\Views;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides an add to cart element.
 *
 * Used to display the add to cart message.
 *
 * Usage example:
 *
 * @code
 * $build['commerce_add_to_cart_confirmation_message'] = [
 *   '#type' => 'commerce_add_to_cart_confirmation_message',
 * ];
 * @endcode
 *
 * @RenderElement("commerce_add_to_cart_confirmation_message")
 */
class AddToCartMessage extends RenderElement implements ContainerFactoryPluginInterface {

  /**
   * The render placeholder generator.
   *
   * @var \Drupal\Core\Render\PlaceholderGeneratorInterface
   */
  protected PlaceholderGeneratorInterface $renderPlaceholderGenerator;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self {
    $instance = new static($configuration, $plugin_id, $plugin_definition);
    $instance->renderPlaceholderGenerator = $container->get('render_placeholder_generator');
    return $instance;
  }

  /**
   * {@inheritdoc}
   *
   * Generate the placeholder in a #pre_render callback, because the hash salt
   * needs to be accessed, which may not yet be available when this is called.
   */
  public function getInfo(): array {
    return [
      '#pre_render' => [
        [$this , 'generatePlaceholder'],
      ],
      '#include_fallback' => FALSE,
    ];
  }

  /**
   * A #pre_render callback to generate a placeholder.
   *
   * @param array $element
   *   A render array.
   *
   * @return array
   *   The updated render array containing the placeholder.
   */
  public function generatePlaceholder(array $element): array {
    $build = [
      '#lazy_builder' => [static::class . '::renderMessage', []],
      '#create_placeholder' => TRUE,
    ];

    // Directly create a placeholder as we need this to be placeholdered
    // regardless if this is a POST or GET request.
    // @todo remove this when https://www.drupal.org/node/2367555 lands.
    $build = $this->renderPlaceholderGenerator->createPlaceholder($build);

    if ($element['#include_fallback']) {
      return [
        'fallback' => [
          '#markup' => '<div data-drupal-add-to-cart-message-fallback class="hidden"></div>',
        ],
        'message' => $build,
      ];
    }
    return $build;
  }

  /**
   * The #lazy_builder callback; replaces placeholder with message.
   *
   * @return array
   *   A render array containing the message.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public static function renderMessage(): array {
    $render = [];

    /** @var \Drupal\commerce_add_to_cart_confirmation\CartConfirmationManagerInterface $cart_confirmation_manager */
    $cart_confirmation_manager = \Drupal::service('commerce_add_to_cart_confirmation.manager');
    $cart_item_info = $cart_confirmation_manager->getCartItemInfo();
    if (empty($cart_item_info['order_item_id'])) {
      return [];
    }

    $view = Views::getView('confirm_message_product_display');
    if (!$view) {
      // The optional view is missing. Log a warning.
      \Drupal::service('commerce_add_to_cart_confirmation.logger')->warning('The "Commerce add to cart confirmation" view is missing.');
      return [];
    }

    $view->setDisplay('default');
    $view->setArguments([$cart_item_info['order_item_id']]);

    $messageTitle = $view->getTitle();

    /** @var \Drupal\commerce_order\Entity\OrderItemInterface|null $order_item */
    $order_item = \Drupal::entityTypeManager()->getStorage('commerce_order_item')->load($cart_item_info['order_item_id']);

    $message = [
      '#theme' => 'commerce_add_to_cart_confirmation',
      '#title' => $order_item?->getTitle() ?? t('Your item')->render(),
      '#message' => $view->render(),
      '#order_item' => $order_item,
      '#quantity' => $cart_item_info['quantity'] ?? NULL,
    ];

    $render['#attached']['drupalSettings']['commerce_add_to_cart_confirmation']['content'] = \Drupal::service('renderer')->render($message);
    $render['#attached']['drupalSettings']['commerce_add_to_cart_confirmation']['title'] = $messageTitle;
    return $render;
  }

}
